/*
 * eth_ecat_datagram.cpp
 *
 *  Created on: May 2, 2023
 *      Author: algin
 */

#include <ethernet_industry/eth_ecat_datagram.hpp>

namespace free_rtos {

namespace datagram {

int32_t EcatTelegram::Process(uint8_t *p_data, uint32_t len) {
    buffer_in_.length = len + sizeof(TEthFrameHeader);

    memcpy(buffer_in_.data, p_data - sizeof(TEthFrameHeader), buffer_in_.length);

    rx_sem_.post();

    return 0;
}

void EcatTelegram::pack(IEcatDatagram& first) {
    TEthFrameHeader *p_eth_hdr = new(buffer_out_.data) TEthFrameHeader{ {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF},
                                                                        {0x01, 0x01, 0x01, 0x01, 0x01, 0x01},
                                                                        ETH_PROT_ECAT_LE};
    TEcatFrameHeader *p_hdr = new(buffer_out_.data + sizeof(TEthFrameHeader)) TEcatFrameHeader{ .bits.length = 0,
                                                                                                .bits.type = static_cast<uint16_t>(ec_network::PROTOCOL_TYPE)};
    uint8_t *p_datagram_first = buffer_out_.data + sizeof(TEthFrameHeader) + sizeof(TEcatFrameHeader);
    uint8_t *p_datagram_last = p_datagram_first;
    IEcatDatagram *next = &first;

    while(next != nullptr) {
        p_datagram_last = next->pack(p_datagram_last);
        next = next->get_next();
    }

    p_hdr->bits.length = p_datagram_last - p_datagram_first;
    buffer_out_.length = sizeof(TEthFrameHeader) + sizeof(TEcatFrameHeader) + p_hdr->bits.length;
}

void EcatTelegram::unpack(IEcatDatagram& first) {
    TEthFrameHeader *p_eth_hdr = reinterpret_cast<TEthFrameHeader*>(buffer_in_.data);
    TEcatFrameHeader *p_hdr = reinterpret_cast<TEcatFrameHeader*>(buffer_in_.data + sizeof(TEthFrameHeader));
    uint8_t *p_datagram_first = buffer_in_.data + sizeof(TEthFrameHeader) + sizeof(TEcatFrameHeader);
    uint8_t *p_datagram_last = p_datagram_first;
    IEcatDatagram *next = &first;

    while(next != nullptr) {
        p_datagram_last = next->unpack(p_datagram_last);
        next = next->get_next();
    }
}

void EcatTelegram::transfer(IEcatDatagram& first) {
    pack(first);
    bool stat = tx_flow_.send(port_id_, buffer_out_.data, buffer_out_.length);

    if(stat == false) {
        return;
    }

    rx_sem_.pend();
    unpack(first);
}

}

}
